# ./src/updates_cfg.py 
import configparser 
from toml_cfg_tool.src.bkup import backup_file
from toml_cfg_tool.src.color_codes import BOLD, CYAN, ORANGE, END
from toml_cfg_tool.src.print_colors import print_two_colors

def update_setup_cfg(file_path, repo_url, updates, dry_run=False, backup=False):
    
    if "github.com" in repo_url:
        config = configparser.ConfigParser()
        config.read(file_path)
        config['metadata']['url'] = repo_url
        with open(file_path, 'w') as f:
            config.write(f)
    
    if backup:
        backup_file(file_path)
    
    config = configparser.ConfigParser()
    config.read(file_path)
    
    changed = False
    
    if 'metadata' not in config:
        config['metadata'] = {}
    
    metadata = config['metadata']
    if 'author' in updates:
        metadata['author'] = updates['author']
        changed = True

    if 'name' in updates:
        metadata['name'] = updates['name']
        changed = True
    
    if 'version' in updates:
        metadata['version'] = updates['version']
        changed = True
            
    if 'description' in updates:
        metadata['description'] = updates['description']
        changed = True
    
    if 'license' in updates:
        metadata['License-Expression'] = updates['license']
        changed = True

    if 'name' in updates:
        if 'options.entry_points' not in config:
            config['options.entry_points'] = {}
            
        options_entry_points = config['options.entry_points']
        options_entry_points['console_scripts'] = f"{updates['name']} = dir_with_entry_script.entry_script_name:main"
        changed = True
        
    
    if 'options' not in config:
        config['options'] = {}
    
    options = config['options']

    if 'requires-python' in updates:
        options['python_requires'] = updates['requires-python']
        changed = True
        
        print_two_colors(BOLD, CYAN, "Updating setup.cfg [options] python_requires to:", updates['requires-python'])
    
    if changed:
        if dry_run:
            print("{BOLD}Dry run enabled. No changes made to setup.cfg.{END}\n")
        else:
            try:
                with open(file_path, 'w') as configfile:
                    config.write(configfile)
                # loop over items changed to True and print them with print_two_colors 
                for key, value in updates.items():
                    if key in ['author', 'name', 'version', 'description', 'license', 'requires-python']:
                        print_two_colors(BOLD, CYAN, f"Updated setup.cfg [metadata] {key} to:", value)
            except Exception as e:
                print(f"{ORANGE}Error updating setup.cfg:{END}{BOLD} {e}{END}\n")
    else:
        print(f"{BOLD}No changes needed in setup.cfg{END}\n")



# ./src/bkup.py
import os
import shutil
from toml_cfg_tool.src.color_codes import BOLD, CYAN, LINK, ORANGE
from toml_cfg_tool.src.print_colors import print_two_colors

def backup_file(file_path):
    if file_path.endswith(".*"):
        file_path = file_path[:-2]
    backup_path = f"{file_path}.bak"
    if os.path.exists(backup_path):
        print_two_colors(ORANGE, LINK, "Backup file already exists:\n", backup_path)
        return
    try:
        shutil.copy(file_path, backup_path)
        print_two_colors(BOLD, CYAN, "Created backup for ", file_path)
    except Exception:
        print_two_colors(ORANGE, LINK, "Error creating backup for ", file_path)



# ./toml_cfg_tool/src/updates_toml.py 
import toml 
from toml_cfg_tool.src.bkup import backup_file
from toml_cfg_tool.src.color_codes import BOLD, CYAN, ORANGE
from toml_cfg_tool.src.print_colors import print_two_colors


def update_pyproject_toml(file_path, repo_url, updates, dry_run=False, backup=False):
    
    if "github.com" in repo_url:
        config = toml.load(file_path)
        config['project']['urls']['Homepage'] = repo_url
        with open(file_path, 'w') as f:
            toml.dump(config, f)
    
    if backup:
        backup_file(file_path)

    try:
        with open(file_path, 'r') as f:
            config = toml.load(f)
    except toml.TomlDecodeError as e:
        print_two_colors(ORANGE, BOLD, f"Error parsing {file_path}:", e)
        return
    except Exception as e:
        print_two_colors(ORANGE, BOLD, f"Failed to read {file_path}:", e)
        return

    project = config.get('project', {})
    changed = False

    if 'author' in updates:
        if 'authors' not in project:
            print_two_colors(ORANGE, BOLD, "Authors section not found in pyproject.toml", "Creating new section.")
            project['authors'] = {'name': updates['author'], 'email': 'your_email@example.com'}
        else:
            project['authors'][0]['name'] = updates['author']
        changed = True

    if 'name' in updates:
        project['name'] = updates['name']
        if 'scripts' in project:
            script_name = updates['name'] + '.' + updates['name'] + ':main'
            print(project['scripts'])
            project['scripts'] = {updates['name']: script_name}
        changed = True

    if 'version' in updates:
        project['version'] = updates['version']
        changed = True
    if 'description' in updates:
        project['description'] = updates['description']
        changed = True
    if 'requires-python' in updates:
        project['requires-python'] = updates['requires-python']
        changed = True
    if 'license' in updates:
        if isinstance(project.get('license', None), dict):
            project['license']['text'] = updates['license']
        else:
            project['license'] = {'text': updates['license']}
        changed = True

    if changed:
        config['project'] = project
        if dry_run:
            print_two_colors(ORANGE, BOLD, "Dry Run:", "Changes not written to pyproject.toml.")
        else:
            try:
                with open(file_path, 'w') as f:
                    toml.dump(config, f)
                # loop over the changes and print them with print_two_colors and their keys and values 
                for key, value in updates.items():
                    print_two_colors(BOLD, CYAN, f"Updated pyproject.toml [{key}]:", value)
            except Exception as e:
                print_two_colors(ORANGE, BOLD, "Failed to update pyproject.toml:", e)
    else:
        print_two_colors(ORANGE, BOLD, "No changes made to pyproject.toml.", "")




import os

def template_text():
    SETUP_CFG_TEMPLATE = f"""\
[metadata]
name = your_project_name
version = 1.0.0 
author = {os.environ.get('USER', 'Your Name')}
author_email = your_email@example.com 
description = A one line description of your project
long_description = file: README.md 
long_description_content_type = text/markdown
url = https://github.com/yourusername/your-repo
License-Expression = MIT
classifiers =
    Programming Language :: Python :: 3 
    License :: OSI Approved :: MIT License 
    Operating System :: OS Independent

[options]
packages = find:
include_package_data = True
python_requires = >=3.7
# install_requires =
#     required_dependency1
#     required_dependency2

[options.entry_points]
console_scripts =
    project_name = dir_with_entry_script.entry_script_name:main
    """

    PYPROJECT_TOML_TEMPLATE = """\
[build-system]
requires = ["setuptools>=61.0", "wheel"]
build-backend = "setuptools.build_meta"

[project]
name = "your_project_name"
version = "1.0.0"
description = "A one line description of your project"
readme = "README.md"
requires-python = ">=3.7"
# dependencies = [ "required_dependency1", "required_dependency2" ]
[[project.authors]]
name = "Your Name"
email = "your_email@example.com"
[project.license]
text = "MIT"
[project.urls]
Homepage = "https://github.com/yourusername/your-repo"
[project.scripts]
project_name = "dir_with_entry_script.entry_script_name:main"

[tool.setuptools.packages.find]
where = ["."]
exclude = ["tests*"]
    """
    templates = {
        'SETUP_CFG_TEMPLATE': SETUP_CFG_TEMPLATE,
        'PYPROJECT_TOML_TEMPLATE': PYPROJECT_TOML_TEMPLATE
    }
    return templates



import os
import sys
import configparser
import toml
import requests
from pathlib import Path

PACKAGE_NAME = "my_package"

def find_package_name():
    cwd = Path.cwd()
    setup_cfg = cwd / 'setup.cfg'
    pyproject_toml = cwd / 'pyproject.toml'
    if os.path.exists(setup_cfg):
        config = configparser.ConfigParser()
        config.read(setup_cfg)
        if 'metadata' in config and 'name' in config['metadata']:
            package_name = config['metadata']['name']

    if os.path.exists(pyproject_toml):
        try:
            with open(pyproject_toml, 'r') as file:
                pyproject_data = toml.load(file)
                package_name = pyproject_data.get('project', {}).get('name')
        except (FileNotFoundError, toml.TomlDecodeError):
            pass

    if not os.path.exists(pyproject_toml) and not os.path.exists(setup_cfg):
        package_name = PACKAGE_NAME

    return package_name

def get_pypi_version(package_name):
    url = f"https://pypi.org/pypi/{package_name}/json"
    print(f"Fetching version from PyPI: {url}")
    try:
        response = requests.get(url)
        response.raise_for_status()
        data = response.json()
        return data['info']['version']
    except requests.exceptions.RequestException as e:
        print(f"Error fetching version from PyPI: {e}")
        return None

def get_setup_cfg_version():
    config = configparser.ConfigParser()
    config.read('setup.cfg')

    if 'metadata' in config and 'version' in config['metadata']:
        return config['metadata']['version']
    return None

def get_pyproject_toml_version():
    try:
        with open('pyproject.toml', 'r') as file:
            pyproject_data = toml.load(file)
            return pyproject_data.get('project', {}).get('version')
    except (FileNotFoundError, toml.TomlDecodeError):
        return None

def main():
    package_name = find_package_name()

    pypi_version = get_pypi_version(package_name)

    setup_cfg_version = get_setup_cfg_version()
    pyproject_toml_version = get_pyproject_toml_version()

    if setup_cfg_version and setup_cfg_version != pypi_version:
        print(f"Version changed in setup.cfg: {pypi_version} -> {setup_cfg_version}")
        sys.exit(0)

    if pyproject_toml_version and pyproject_toml_version != pypi_version:
        print(f"Version changed in pyproject.toml: {pypi_version} -> {pyproject_toml_version}")
        sys.exit(0)

    print("No version changes detected.")
    sys.exit(1)

if __name__ == "__main__":
    main()



# ./src/creation.py 
import os
import shutil
import subprocess
import toml
import configparser
import glob
from pathlib import Path
from toml_cfg_tool.src.config import template_text 
from toml_cfg_tool.src.color_codes import BOLD, CYAN, LINK, ORANGE
from toml_cfg_tool.src.print_colors import print_two_colors

SETUP_CFG_TEMPLATE = template_text()['SETUP_CFG_TEMPLATE']
PYPROJECT_TOML_TEMPLATE = template_text()['PYPROJECT_TOML_TEMPLATE']

def get_github_repo_url():
    try:
        result = subprocess.run(
            ["git", "config", "--get", "remote.origin.url"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            check=True
        )
        git_url = result.stdout.strip()
        return git_url if result.stdout else "No GitHub repository found."
    except subprocess.CalledProcessError:
        return "Not a Git repository."

def create_setup_cfg_template(file_path, dry_run=False):
    if os.path.exists(file_path):
        print_two_colors(BOLD, ORANGE, "setup.cfg already exists.", "Skipping template creation.")
        return
    if dry_run:
        print_two_colors(BOLD, CYAN, "[Dry Run]", "Template not written.")
        return
    try:
        with open(file_path, 'w') as f:
            f.write(SETUP_CFG_TEMPLATE)
        print_two_colors(BOLD, LINK, "Template setup.cfg created successfully.", file_path)
    except Exception as e:
        print_two_colors(ORANGE, BOLD, "Failed to create setup.cfg:", e)

    repo_url = get_github_repo_url()
    if "github.com" in repo_url:
        config = configparser.ConfigParser()
        config.read(file_path)
        config['metadata']['url'] = repo_url
        with open(file_path, 'w') as f:
            config.write(f)

def create_pyproject_toml_template(file_path, dry_run=False):
    if os.path.exists(file_path):
        print_two_colors(BOLD, ORANGE, "pyproject.toml already exists.", "Skipping template creation.")
        return
    if dry_run:
        print_two_colors(BOLD, CYAN, "[Dry Run]", "Template not written.")
        return
    try:
        with open(file_path, 'w') as f:
            f.write(PYPROJECT_TOML_TEMPLATE)
        print_two_colors(BOLD, LINK, "Template pyproject.toml created successfully.", file_path)
    except Exception as e:
        print_two_colors(ORANGE, BOLD, "Failed to create pyproject.toml:", e)
   
    repo_url = get_github_repo_url()
    if "github.com" in repo_url:
        config = toml.load(file_path)
        config['project']['urls']['Homepage'] = repo_url
        with open(file_path, 'w') as f:
            toml.dump(config, f)

def create_workflow_files():
    root = Path(__file__).parent.parent
    src = root / "src/github_workflows"
    cwd = Path.cwd()
    github = cwd / ".github"
    workflow_dest = cwd / ".github/workflows"
    script_dest = cwd / ".github/scripts"
    workflow_files = glob.glob(f"{src}/*.yml")
    script_files = glob.glob(f"{src}/*.sh") + glob.glob(f"{src}/*.py")

    # Get the basenames of the current workflow and script files for comparison
    current_workflow_files = [os.path.basename(f) for f in glob.glob(f"{workflow_dest}/*.yml")]
    current_script_files = [os.path.basename(f) for f in glob.glob(f"{script_dest}/*.sh") + glob.glob(f"{script_dest}/*.py")]

    if not os.path.exists(github):
        print_two_colors(BOLD, LINK, "Folder not found:", github)
        return

    if not os.path.exists(workflow_dest):
        print_two_colors(BOLD, LINK, f"Creating folder:", workflow_dest)
        os.makedirs(workflow_dest)

    if not os.path.exists(script_dest):
        print_two_colors(BOLD, LINK, f"Creating folder:", script_dest)
        os.makedirs(script_dest)

    skip_counter = 0

    # Copy workflow files only if they don't already exist
    for file in workflow_files:
        if os.path.basename(file) in current_workflow_files:
            skip_counter += 1
            continue
        print_two_colors(BOLD, LINK, "Copying", file)
        print_two_colors(BOLD, LINK, "to", workflow_dest)
        new_file = workflow_dest / os.path.basename(file)
        shutil.copyfile(file, new_file)

    # Copy script files only if they don't already exist
    for file in script_files:
        if os.path.basename(file) in current_script_files:
            skip_counter += 1
            continue
        print_two_colors(BOLD, LINK, "Copying", file)
        print_two_colors(BOLD, LINK, "to", script_dest)
        new_file = script_dest / os.path.basename(file)
        shutil.copyfile(file, new_file)

    if skip_counter == len(workflow_files) + len(script_files):
        print_two_colors(BOLD, ORANGE, "All workflow and script files already exist.", "Skipping copy.")



#!/usr/bin/env python3

import os
import sys
from toml_cfg_tool.src.cli import parse_arguments
from toml_cfg_tool.src.bkup import backup_file
from toml_cfg_tool.src.creation import create_setup_cfg_template, create_pyproject_toml_template, create_workflow_files, get_github_repo_url
from toml_cfg_tool.src.color_codes import BOLD, CYAN
from toml_cfg_tool.src.print_colors import print_two_colors
from toml_cfg_tool.src.updates_cfg import update_setup_cfg
from toml_cfg_tool.src.updates_toml import update_pyproject_toml
from toml_cfg_tool.src.contrib import create_contrib_file
from toml_cfg_tool.src.reader import print_updateable_values 
from pathlib import Path

def main():
    args = parse_arguments()

    repo_url = get_github_repo_url()
    print_two_colors(BOLD, CYAN, "GitHub repository found, updating cfg and toml files to match:", repo_url)
    
    root_dir = Path.cwd()
    if args.show:
        print_updateable_values(root_dir)
        sys.exit(0)

    if args.create_contrib:
        create_contrib_file()

    if args.backup:
        file_path = args.backup 
        backup_file(file_path)

    updates = {}
    if args.author:
        updates['author'] = args.author
    if args.name:
        updates['name'] = args.name
    if args.version:
        updates['version'] = args.version
    if args.description:
        updates['description'] = args.description
    if args.requires_python:
        updates['requires-python'] = args.requires_python
    if args.license:
        updates['license'] = args.license

    if not args.workflow_files:
        pass
    else:
        create_workflow_files()

    if not updates and not args.create_templates:
        sys.exit(0)
    
    files = ["setup.cfg", "pyproject.toml"]
    
    for file in files:
        if args.create_templates:
            if file.endswith(".cfg"):
                create_setup_cfg_template(file, dry_run=args.dry_run)
            elif file.endswith(".toml"):
                create_pyproject_toml_template(file, dry_run=args.dry_run)
        
        if os.path.exists(file) and updates:
            print(f"--- Processing {file} ---")
            
            if file.endswith(".cfg"):
                update_setup_cfg(file, repo_url, updates, dry_run=args.dry_run, backup=args.backup)
            
            elif file.endswith(".toml"):
                update_pyproject_toml(file, repo_url, updates, dry_run=args.dry_run, backup=args.backup)
        
        elif not os.path.exists(file):
            if not args.create_templates:
                print(f"File not found: {file}\n")

if __name__ == "__main__":
    main()




# ./src/cli.py 
import argparse

def parse_arguments():
    parser = argparse.ArgumentParser(description="Update metadata fields in setup.cfg and pyproject.toml or create template files, including GitHub Actions workflow files.")
    
    parser.add_argument("--author", type=str, help="Update the project author.")
    parser.add_argument("--name", type=str, help="Update the project name.")
    parser.add_argument("--version", type=str, help="Update the project version.")
    parser.add_argument("--description", type=str, help="Update the project description.")
    parser.add_argument("--requires-python", type=str, help="Update the required Python version.")
    parser.add_argument("--license", type=str, help="Update the project license.")
    parser.add_argument("--workflow_files", action="store_true", help="Add GitHub Actions workflow files, and scripts.")
    parser.add_argument("--create-templates", action="store_true", help="Create template setup.cfg and pyproject.toml files if they do not exist.")
    parser.add_argument("--create-contrib", action="store_true", help="Create a CONTRIBUTING.md file if it does not exist.")
    parser.add_argument("--backup", action="store_true", help="Backup existing configuration files before making changes.")
    parser.add_argument("--dry-run", action="store_true", help="Show changes without writing to files.")
    parser.add_argument("--show", action="store_true", help="Show the current configurations that the script can update.")
    
    return parser.parse_args()




import toml 
from toml_cfg_tool.src.print_colors import print_two_colors
from toml_cfg_tool.src.color_codes import ORANGE, BOLD

def read_updateable_toml_values(file_path):
    file_path = file_path / 'pyproject.toml'
    try:
        with open(file_path, 'r') as f:
            config = toml.load(f)
    except toml.TomlDecodeError as e:
        print_two_colors(ORANGE, BOLD, f"Error parsing {file_path}:", e)
        return
    except Exception as e:
        print_two_colors(ORANGE, BOLD, f"Failed to read {file_path}:", e)
        return

    project = config.get('project', {})
    values = {
        'author': project.get('authors', [{}])[0].get('name', ''),
        'name': project.get('name', ''),
        'version': project.get('version', ''),
        'description': project.get('description', ''),
        'requires-python': project.get('requires-python', ''),
        'license': project.get('license', {}).get('text', '')
    }
    return values

def print_updateable_values(file_path):
    values = read_updateable_toml_values(file_path)
    if values:
        print('Updateable values:')
        for key, value in values.items():
            print_two_colors(BOLD, ORANGE, f'{key}:', value)
    else:
        print('No updateable values found.')



import os
import re
import subprocess
from pathlib import Path
from toml_cfg_tool.src.color_codes import BOLD, LINK
from toml_cfg_tool.src.print_colors import print_two_colors

def find_delete_line(file_path, delete_line, new_text=None):
    path = Path(file_path)
    text = path.read_text()
    if new_text:
        text = new_text
    new_lines = [line for line in text.split("\n") if delete_line not in line]
    updated_text = "\n".join(new_lines)
    path.write_text(updated_text)

def loop_replace_text(file_path, old_text_list, new_text_list):
    path = Path(file_path)
    text = path.read_text()
    for i in range(len(old_text_list)):
        new_text = re.sub(old_text_list[i], new_text_list[i], text)
        text = new_text

    if os.path.exists(file_path):
        pass
    else:
        with open(file_path, 'w') as f:
            f.write(text)

    return text

def get_github_repo_info():
    try:
        result = subprocess.run(
            ["git", "config", "--get", "remote.origin.url"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
            check=True
        )
        url = result.stdout.strip()

        match = re.search(r'github\.com[:/](?P<username>[^/]+)/(?P<repo>[^/]+)\.git', url)
        
        if match:
            github_username = match.group("username")
            repo_name = match.group("repo")
            return repo_name, github_username
        else:
            return None, None
    except subprocess.CalledProcessError:
        return None, None

def create_contrib_file():
    delete_line = "### Delete this line"
    src_path = Path(__file__).parent.parent
    file_path = src_path / "src/github_workflows/CONTRIBUTING.md"
    subprocess.run(["cp", file_path, ".github/CONTRIBUTING.md"])
    repo_name, github_username = get_github_repo_info()
    if repo_name is None:
        repo_name = "RepoName"
    if github_username is None:
        github_username = "GitHubUsername"
    old_text_list = ["RepoName", "GitHubUsername"]
    new_text_list = [repo_name, github_username]
    cwd = Path.cwd()
    github_dir = cwd / ".github"
    if github_dir.exists():
        print_two_colors(BOLD, LINK, "Creating CONTRIBUTING.md in .github directory", github_dir)
        github_dir / "CONTRIBUTING.md"
        github_dir.mkdir(exist_ok=True)
        github_file_path = github_dir / "CONTRIBUTING.md"
        text = loop_replace_text(github_file_path, old_text_list, new_text_list)
        find_delete_line(github_file_path, delete_line, text)
    else:
        text = loop_replace_text(file_path, old_text_list, new_text_list)
        find_delete_line(file_path, delete_line, text)
        print("No .github directory found at", cwd)
